/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import java.util.List;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.WindowResizeListener;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.animation.client.Animation;
import com.google.gwt.core.client.Duration;

/**
 * Abstract super class for the panels which appear in the repository view.
 * Provides common ways of constructing tabs and content.
 * 
 */
public abstract class AbstractRepositoryPanel extends FlowPanel {
    
    protected RefreshablePanel content = null;
    private FlowPanel bbi;
    private FlowPanel bb;

    /**
     * Method to display a title bar for the panel.
     * Usually this contains a list of sub-tabs and uses {@link PipesMenu} to display green buttons.
     *
     * @return A widget to use as title panel.
     */
    protected abstract Widget getTitleWidget();

    /**
     * Override the method to display additional title bar panel.
     * This one is usually used to display sorting order drop-down and filtering pipes.
     *
     * @see AbstractRepositoryPanel#getContentsWidget AbstractListPanel.getContentsWidget() for sample implementation.
     *
     * @return A {@link RefreshablePanel} to display second title bar panel.
     */
    public abstract RefreshablePanel getContentsWidget();
    
    protected abstract Widget getTabsWidget();
    
    private FlowPanel content_i3 = new FlowPanel();
    private FlowPanel i3i = new FlowPanel();

    private WindowResizeListener windowResizeListener;

    protected AbstractRepositoryPanel() {
        setWidth("100%");

        windowResizeListener = new WindowResizeListener() {
            public void onWindowResized(int width, int height) {
                updateDimensions(width, height);
            }
        };
    }

    @Override
    protected void onAttach() {
        super.onAttach();
        Window.addWindowResizeListener(windowResizeListener);
    }


    @Override
    protected void onDetach() {
        Window.removeWindowResizeListener(windowResizeListener);
        super.onDetach();
    }

    /**
     * Builds a widget that displays additional actions appropriate to the current pane.
     * @return   the widget with the actions
     */
    public abstract Widget getActionsWidget();
    
    /**
     * Builds a widget that displays information about the current pane.
     * @return  the widget containing the appropriate information
     */
    protected abstract Widget getInformationWidget();
    
    /**
     * Builds a hyperlink that is displayed at the bottom of the information box if additional 
     * help exists for the topic.
     * @return  the hyperlink pointing to the apppropriate content
     */
    protected abstract Widget getInformationLinkWidget();
        
    /**
     * Rebuild and redraw panel contents.
     */
    public void  refresh() {
        refresh(false, false);
    }
    
    public void refresh(boolean animate) {
        refresh(animate, false);
    }

    public void refresh(boolean animate, boolean debug) {
        if (getWidgetCount() > 0) {
            clear();
        }

        FlowPanel left = new FlowPanel();
        left.setStyleName("twLeftRepoPanel");

        Widget title = getTitleWidget();
        if (title != null) {
            if (debug)
                Window.alert("AbstractRepositoryPanel.refresh adding title panel: " + title);
            CommonUtils.addAutomationId(title, "gwtcommon.client.abstractRepositoryPanel.title_0");

            if(animate) {
                animateGrowHeight(title);
            }

            left.add(title);
        }

        Widget tabs = getTabsWidget();
        if (tabs != null) {
            if (debug)
                Window.alert("AbstractRepositoryPanel.refresh adding tabs: " + tabs);
            CommonUtils.addAutomationId(tabs, "gwtcommon.client.abstractRepositoryPanel.tabs_0");
            left.add(tabs);
        }

        Widget contents = createContents();
        if (contents != null) {
            if (debug)
                Window.alert("AbstractRepositoryPanel.refresh adding contents: " + contents);
            CommonUtils.addAutomationId(contents, "gwtcommon.client.abstractRepositoryPanel.contents_0");
            if(title != null) {
                contents.addStyleName("twUnderTitle");
            }
            left.add(contents);
        }
        
        add(left);

        Widget right = createRight();
        if (right != null) {
            if (debug)
                Window.alert("AbstractRepositoryPanel.refresh adding right panel: " + right);
            CommonUtils.addAutomationId(right, "gwtcommon.client.abstractRepositoryPanel.right_0");
            add(right);
        }

        updateDimensionsDeferred();
        
        focusOnDefault();
    }

    private void animateGrowHeight(final Widget widget) {
        final int originalPixelHeight = 35;
        widget.setHeight("0");
        Animation growTitleHeightAnimation = new Animation() {
            protected void onUpdate(double progress) {
                int newHeight = (int)(originalPixelHeight * interpolate(progress));
//                Window.alert(newHeight + "@" + progress);
                widget.setHeight(newHeight + "px");
            }
        };
        growTitleHeightAnimation.run(1000, Duration.currentTimeMillis() + 500);
    }


    protected Widget createContents() {
        FlowPanel cb = new FlowPanel();
        CommonUtils.addAutomationId(cb, "gwtcommon.client.abstractRepositoryPanel.flowPanel_cb");
        cb.setStyleName("cb");
        
        FlowPanel bt = new FlowPanel();
        CommonUtils.addAutomationId(bt, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bt");
        bt.setStyleName("bt");
        bt.add(new FlowPanel());
        cb.add(bt);
        
        FlowPanel i1 = new FlowPanel();
        CommonUtils.addAutomationId(i1, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i1");
        i1.setStyleName("i1");
        cb.add(i1);
        
        FlowPanel i2 = new FlowPanel();
        CommonUtils.addAutomationId(i2, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i2");
        i2.setStyleName("i2");
        i1.add(i2);
        
        content_i3 = new FlowPanel();
        CommonUtils.addAutomationId(content_i3, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i3");
        content_i3.setStyleName("i3");
        i2.add(content_i3);

        content = getContentsWidget();
        if (content != null) {
            content_i3.clear();
            content_i3.add(content);
        }

        bb = new FlowPanel();
        CommonUtils.addAutomationId(bb, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bb");
        bb.setStyleName("bb");
        bb.add(new FlowPanel());
        cb.add(bb);
        
        return cb;
    }
    
    public void refreshContents() {
        if (content != null) {
            content.refresh();
        }
    }

    protected Widget createRight() {
        FlowPanel right = new FlowPanel();
        CommonUtils.addAutomationId(right, "gwtcommon.client.abstractRepositoryPanel.flowPanel_right");
        right.setStyleName("right");

        SimplePanel shadowHolder = new SimplePanel();
        shadowHolder.addStyleName("verticalShadow");
        right.add(shadowHolder);

        FlowPanel cb2 = new FlowPanel();
        CommonUtils.addAutomationId(cb2, "gwtcommon.client.abstractRepositoryPanel.flowPanel_cb2");
        cb2.setStyleName("cb2");
        right.add(cb2);
        
        FlowPanel bt = new FlowPanel();
        CommonUtils.addAutomationId(bt, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bt");
        bt.setStyleName("actionbt");
        bt.add(new FlowPanel());
        cb2.add(bt);        
        
        FlowPanel i1 = new FlowPanel();
        CommonUtils.addAutomationId(i1, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i1");
        i1.setStyleName("actioni1");
        cb2.add(i1);
        
        FlowPanel i2 = new FlowPanel();
        CommonUtils.addAutomationId(i2, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i2");
        i2.setStyleName("actioni2");
        i1.add(i2);
        
        FlowPanel i3 = new FlowPanel();
        CommonUtils.addAutomationId(i3, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i3");
        i3.setStyleName("actioni3");
        i2.add(i3);
        
        // build and add the widget containing other actions that can
        // be performed from this pane.
        Widget actions = getActionsWidget();
        if (actions != null) {        
            i3.add(actions);
        }
        else {
            // we have no actions so remove the actions box completely
            right.remove(cb2);
        }
        
        FlowPanel bba = new FlowPanel();
        CommonUtils.addAutomationId(bba, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bb");
        bba.setStyleName("bba");
        bba.add(new FlowPanel());
        cb2.add(bba);

        FlowPanel cb2a = new FlowPanel();
        cb2a.setStyleName("cb2a");
        cb2a.add(new FlowPanel());
        right.add(cb2a);

        FlowPanel cb3 = new FlowPanel();
        CommonUtils.addAutomationId(cb3, "gwtcommon.client.abstractRepositoryPanel.flowPanel_cb3");
        cb3.setStyleName("cb3");
        right.add(cb3);
        
        FlowPanel bti = new FlowPanel();
        CommonUtils.addAutomationId(bti, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bti");
        bti.add(new FlowPanel());
        bti.setStyleName("bti");
        cb3.add(bti);

        FlowPanel i1i = new FlowPanel();
        CommonUtils.addAutomationId(i1i, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i1i");
        i1i.setStyleName("i1i");
        cb3.add(i1i);
        
        FlowPanel i2i = new FlowPanel();
        CommonUtils.addAutomationId(i2i, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i2i");
        i2i.setStyleName("i2i");
        i1i.add(i2i);
        
        // build spot for the information widget
        i3i = new FlowPanel();
        CommonUtils.addAutomationId(i3i, "gwtcommon.client.abstractRepositoryPanel.flowPanel_i3i");
        i3i.setStyleName("i3i");
        i2i.add(i3i);
        
        // build information panel and add it to the appropriate spot
        refreshInformation();

        bbi = new FlowPanel();
        CommonUtils.addAutomationId(bbi, "gwtcommon.client.abstractRepositoryPanel.flowPanel_bbi");
        bbi.add(new FlowPanel());
        bbi.setStyleName("bbi");
        cb3.add(bbi);

        return right;
    }

    public void refreshInformation() {
        Widget info = getInformationWidget();
        if (info != null) {
            i3i.clear();
            i3i.add(info);
            
            Widget link = getInformationLinkWidget();
            if (link != null){
                i3i.add(link);
            }
        }
    }

    public void goToPage(List subList, AsyncCallback callback) {
        // by default do nothing
        callback.onSuccess(this);
    }


    protected final void updateDimensionsDeferred() {
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                updateDimensions();
            }
        });
    }

    protected final void updateDimensions() {
        updateDimensions(Window.getClientWidth(), Window.getClientHeight());
    }

    public void updateDimensions(int clientWidth, int clientHeight) {
        if(bbi != null) {
            int bbiHeight = clientHeight - bbi.getAbsoluteTop() - 2;
            if(bbiHeight < 0) {
                bbiHeight = 0;
            }
            bbi.setHeight(bbiHeight + "px");
        }
    }
    
    /*
     * Forces the page to focus on a default text box on load/refresh; most repository pages will not need
     * to override, as they do not have a text box.
     */
    public void focusOnDefault() {
    }

    /**
     * Sets panel opacity.
     * @param opacity double Opacity level. 
     */
    public void setOpacity(final double opacity) {
        this.getElement().getStyle().setProperty("opacity", "" + opacity);
    }
}
