/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.rpc.StatusCodeException;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DisclosurePanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.ScrollPanel;

import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;

/**
 * The class to display exception stack trace and error message in a pop-up.
 *
 * @author bcampbell
 * @author Slava Nadvorny vnadvorny
 * @author Austin Arnold aarnold
 */
public class ExceptionPopup extends AbstractRepositoryPopup {

    /**
     * There is only one popup.
     */
    private static ExceptionPopup instance;

    /**
     * The list of visible exception pairs.
     */
    private static final Set<ExceptionPair> visibleExceptionPairs = new HashSet<ExceptionPair>();

    /**
     * The panel that holds the exceptions.
     */
    private FlowPanel content;

    /**
     * Show the exception in a popup window.
     * 
     * @param throwable Exception to log.
     */
    public static void showException(Throwable throwable) {
        showException(throwable, null);
    }

    /**
     * Show the exception in a popup window with message.
     *
     * @param throwable Exception to log.
     * @param userMessage - if you want to override the message that is part of the exception
     */
    public static void showException(Throwable throwable, String userMessage) {
        if (instance == null) {
            // No popup, create one and show it
            instance = new ExceptionPopup(throwable, userMessage);
            instance.center();
            instance.show();
        } else {
            // Open popup already, add the pair to the list
            instance.addExceptionPair(throwable, userMessage);
        }
    }

    /**
     * This will add an exception pair to a current dialog as long as it is unique.
     */
    private void addExceptionPair(Throwable throwable, String userMessage) {
        ExceptionPair exceptionPair = new ExceptionPair(throwable, userMessage);

        // Skip identical exceptions from being added multiple times.
        if (!visibleExceptionPairs.add(exceptionPair)) {
            return;
        }

        FlowPanel main = new FlowPanel();
        main.setWidth("375px");
        main.getElement().getStyle().setProperty("height", "auto");

        String message = "";
        if(userMessage != null) {
            message += userMessage;
        }
        else if(throwable.getMessage() != null) {
            message += throwable.getMessage();
        }
        HTML userMessageLabel = new HTML(message);
        main.add(userMessageLabel);

        DisclosurePanel dPanel = new DisclosurePanel("More Details...", false);
        dPanel.setWidth("100%");
        dPanel.setAnimationEnabled(true);

        TextBox fullExceptionTextBox = new TextBox();
        fullExceptionTextBox.setReadOnly(true);
        fullExceptionTextBox.setWidth("100%");
        fullExceptionTextBox.setHeight("300px");
        fullExceptionTextBox.setText(getStackTraceString(throwable));
        dPanel.add(fullExceptionTextBox);

        main.add(dPanel);
        content.add(main);
    }

    protected ExceptionPopup(Throwable throwable, String userMessage) {
        super();
        rebuildPanel();

        addExceptionPair(throwable, userMessage);
    }

    protected List<LibraryButton> getButtons() {
        List<LibraryButton> buttons = new ArrayList<LibraryButton>();
        LibraryButton closePopups = new LibraryButton(CommonUtils.getCommonMessages().errorCloseAllMessages());
        closePopups.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                close();
            }
        });
        buttons.add(closePopups);

        return buttons;
    }

    protected Widget getContents() {
        ScrollPanel scroll = new ScrollPanel();
        scroll.setPixelSize(400, 200);
        content = new FlowPanel();
        scroll.add(content);
        return scroll;
    }

    protected String getTitleText() {
        return CommonUtils.getCommonMessages().errorDialogTitle();
    }

    @Override
    protected void onClose() {
        visibleExceptionPairs.clear();
        instance = null;
        super.onClose();
    }

    /**
    * Defines a custom format for the stack trace as String.
    */
    public static String getStackTraceString(Throwable aThrowable) {
        final StringBuilder result = new StringBuilder();
        if (aThrowable instanceof StatusCodeException) {
            result.append(Integer.toString(((StatusCodeException) aThrowable).getStatusCode()));
            result.append("\n");
        }
        result.append(aThrowable.toString());
        result.append("\n");

        //add each element of the stack trace
        for (StackTraceElement element : aThrowable.getStackTrace()) {
            result.append(element);
            result.append("\n");
        }
        return result.toString();
    }

    /**
     * Closes exception dialog.
     */
    public static void closeException() {
        instance.close();
    }

    /**
     * This is a pairing of the throwable and the user message.
     */
    private class ExceptionPair {
        /**
         * The exception caught to display in the pop-up.
         */
        private Throwable throwable;

        /**
         * Custom message to show along with exception stack trace.
         */
        private String userMessage;

        public ExceptionPair(Throwable throwable, String userMessage) {
            this.throwable = throwable;
            this.userMessage = userMessage;
        }

        /**
         * Equal if string representations of exception are equal.
         */
        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof ExceptionPair)) return false;

            ExceptionPair that = (ExceptionPair) o;

            if (!getStackTraceString(throwable).equals(getStackTraceString(that.throwable))) return false;
            if (userMessage != null ? !userMessage.equals(that.userMessage) : that.userMessage != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = getStackTraceString(throwable).hashCode();
            result = 31 * result + (userMessage != null ? userMessage.hashCode() : 0);
            return result;
        }
    }


}
